﻿using BenchmarkDotNet.Attributes;
using FlexGridBenchmark.Utils;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Windows.Forms;

#if CLR461
using C1.Win.C1FlexGrid;
#elif CLR5
using C1.Win.FlexGrid;
#endif

namespace FlexGridBenchmark.Benchmarks
{
    public class FilterColumnBenchmarks : BaseBenchmarks
    {
        Data _data = null;

        Form _form = null;
        C1FlexGrid _flexGrid = null;

        int _condition1Parameter = int.MinValue;
        int _condition2Parameter = int.MinValue;

        [GlobalSetup]
        public void GlobalSetup()
        {
            _data = DataGenerator.Generate(RowCount, ColumnCount);

            SortedSet<int> column0Values = new SortedSet<int>();

            DataRowCollection rows = _data.DataTable.Rows;
            foreach (DataRow row in rows)
            {
                column0Values.Add((int)row.ItemArray[0]);
            }

            int column0ValuesCount = column0Values.Count;
            int column0ValuesQuarterCount = column0ValuesCount / 4;
            int column0ValuesThreeQuartersCount = column0ValuesQuarterCount * 3;

            _condition1Parameter = column0Values.ElementAt(column0ValuesQuarterCount);
            _condition2Parameter = column0Values.ElementAt(column0ValuesThreeQuartersCount);
        }

        [GlobalCleanup]
        public void GlobalCleanup()
        {
            _data?.Dispose();
            _data = null;
        }

        [IterationSetup]
        public void IterationSetup()
        {
            _form = new Form() { WindowState = FormWindowState.Maximized };
            _flexGrid = new C1FlexGrid() { Dock = DockStyle.Fill, UseCompatibleTextRendering = false };
            _form.Controls.Add(_flexGrid);

            _flexGrid.DataSource = _data.DataTable;

            int firstDataMapColumnIndex = _flexGrid.Cols.Fixed + _data.FirstDataMapColumnIndex;
            int flexGridColumnCount = _flexGrid.Cols.Count;
            int dataTypeCount = _data.DataTypeCount;
            int dataMapIndex = 0;
            for (int columnIndex = firstDataMapColumnIndex; columnIndex < flexGridColumnCount; columnIndex += dataTypeCount)
            {
                _flexGrid.Cols[columnIndex].DataMap = _data.DataMaps[dataMapIndex];
                dataMapIndex++;
            }

            _form.Show();
            _flexGrid.Refresh();
        }

        [IterationCleanup]
        public void IterationCleanup()
        {
            _flexGrid.Refresh();
            _flexGrid?.Dispose();
            _flexGrid = null;
            _form?.Dispose();
            _form = null;
        }

        [Benchmark]
        public void FilterColumn()
        {
            _flexGrid.AllowFiltering = true;
            ConditionFilter conditionFilter = (_flexGrid.Cols[_flexGrid.Cols.Fixed].Filter as ColumnFilter).ConditionFilter;

            Condition condition1 = conditionFilter.Condition1;
            condition1.Operator = ConditionOperator.GreaterThan;
            condition1.Parameter = _condition1Parameter;

            Condition condition2 = conditionFilter.Condition2;
            condition2.Operator = ConditionOperator.LessThan;
            condition2.Parameter = _condition2Parameter;

            _flexGrid.ApplyFilters();
        }
    }
}
