﻿using BenchmarkDotNet.Attributes;
using FlexGridBenchmark.Utils;
using System.Windows.Forms;

#if CLR461
using C1.Win.C1FlexGrid;
#elif CLR5
using C1.Win.FlexGrid;
#endif

namespace FlexGridBenchmark.Benchmarks
{
    public class GenerateAndAssignDataBenchmarks : BaseBenchmarks
    {
        Data _data = null;

        Form _form = null;
        C1FlexGrid _flexGrid = null;

        [IterationSetup]
        public void IterationSetup()
        {
            _form = new Form() { WindowState = FormWindowState.Maximized };
            _flexGrid = new C1FlexGrid() { Dock = DockStyle.Fill, UseCompatibleTextRendering = false };
            _form.Controls.Add(_flexGrid);
            _form.Show();
            _flexGrid.Refresh();
        }

        [IterationCleanup]
        public void IterationCleanup()
        {
            _flexGrid.Refresh();
            _flexGrid?.Dispose();
            _flexGrid = null;
            _form?.Dispose();
            _form = null;
            _data?.Dispose();
            _data = null;
        }

        [Benchmark]
        public void GenerateAndAssignData()
        {
            _data = DataGenerator.Generate(RowCount, ColumnCount);

            _flexGrid.DataSource = _data.DataTable;

            int firstDataMapColumnIndex = _flexGrid.Cols.Fixed + _data.FirstDataMapColumnIndex;
            int flexGridColumnCount = _flexGrid.Cols.Count;
            int dataTypeCount = _data.DataTypeCount;
            int dataMapIndex = 0;
            for (int columnIndex = firstDataMapColumnIndex; columnIndex < flexGridColumnCount; columnIndex += dataTypeCount)
            {
                _flexGrid.Cols[columnIndex].DataMap = _data.DataMaps[dataMapIndex];
                dataMapIndex++;
            }
        }
    }
}
